// By Phill Leyva
// September 4 / 2001
// Maxim Integrated Products
// *********** LPT Port powered Step-Down DC-DC Controller ***********
// Version = 1.0
//	ICs used in Project
//		U1 = MAX1733 Low-Voltage, Step-Down DC-DC Converter
//		U2 = MAX1841 Low-Voltage, SIM/Smart Card Level Translator
// 		U3 = SN74HC165 8-bit Parallel-Load Shift Register
//		U4 = NC7SZ05, Fairchild UHS Inverter (Open Drain)
//              D1-D3 = BAT54C, Schottky Barrier Diode
//
// Header file section
#include <conio.h>
#include <iostream.h>
#include <dos.h>

// Function Prototypes
int Receive_In(int); 	// Receive serial data from the port
void CLK_Low(void);     // Set Clock bit low
void CLK_High(void);    // Set Clock bit high
void CS_Low(void);      // Set CS\ bit low
void CS_High(void);     // Set CS\ bit high
void Data_Low(void);    // Set Data bit low
void Data_High(void);   // Set Data bit high
void Send_Out(int);	// Optional function to Send out serial data

// Global variables
	int PortData = 0x00; // Initialize LPT port base address data
	int CP_Disable = 0x02; // Control port data, set off
	int CP_Enable = 0x00; // Control port data, set on
	int Input_PortData;  // 8-bit read from LPT port Status address
	unsigned PortAddress = 0x378;	// LPT port base address, 378, 278
	unsigned StatusPort = (PortAddress + 1); // Status port address
	unsigned ControlPort = (PortAddress + 2); // Control port address
	int Out_PortData = 0x00;  //Optional, replace 0xF1 with desired data
				 // to send out

main()
{
// variable declaration
char Choice = '0';              // Menu choice

outp(PortAddress,0x00);  // Setting the LPT data port to off
outp(ControlPort,CP_Disable);  // Setting the LPT control port to off

clrscr();				// clear screen
cout << "---> Enter a menu number <---";
cout << "\n";
cout << "---> Power UP Port Power = 1 \n";
cout << "---> Power DOWN Port Power = 0 \n ";
cout << " \n";
cout << "---> ENABLE the +2.0V Power Supply (MAX1733) = 7 \n";
cout << "---> DISABLE the +2.0V Power Supply (MAX1733) = 8 \n ";
cout << " \n";
cout << "---> READ Data from the Port = 4 \n ";
cout << "---> Optional *** WRITE Data to the Port = 5 \n ";
cout << " \n";
cout << "---> Exit the Program = 9 \n ";
cout << " \n";
cout << "---> Enter Your Selection: \n";

while (Choice != '9')
      { gotoxy(28,13);
      clreol();
      cin >> Choice;

      if (Choice == '1')
	{ gotoxy(20, 17);
	clreol();
	cout << " \n";
	cout << "Turning ON LPT Port Power \n";
	clreol();
	cout << "LPT Port Power = ON \n";
	PortData = 0xFF;
	outp(PortAddress,PortData);  // Turn ON Port
	}

      else if (Choice == '0')
	{ gotoxy(20, 17);
	clreol();
	cout << " \n";
	cout << "Turning OFF LPT Port Power \n";
	clreol();
	cout << "LPT Port Power = OFF \n";
	PortData = 0x00;
	outp(PortAddress,PortData);  // Turn OFF Port
	}

      else if (Choice == '7')
	{ gotoxy(20, 20);
	clreol();
	cout << " \n";
	cout << "Enabling 2V Power Supply \n";
	clreol();
	cout << "MAX1733 2V Power Supply = Enabled \n";
	outp(ControlPort,CP_Enable);  // Enable MAX1733 PS
	}

      else if (Choice == '8')
	{ gotoxy(20, 20);
	clreol();
	cout << " \n";
	cout << "Disabling 2V Power Supply \n";
	clreol();
	cout << "MAX1733 2V Power Supply = Disabled \n";
	outp(ControlPort,CP_Disable);  // Disable MAX1733 PS
	}

      else if (Choice == '4')
	{
	Input_PortData = Receive_In(StatusPort);  // Get port data
	gotoxy(24, 23);
	clreol();
	cout << " \n";
	cout << "The Port input data is " << Input_PortData; // data in decimal format
	clreol();
	cout << "\n";
	}

      else if (Choice == '5')  	// Optional choice, need hardware 1st
	{
	Send_Out(Out_PortData);  // Send out data
	gotoxy(24, 23);
	clreol();
	cout << " \n";
	cout << "Sending out Data to the Port \n";
	clreol();
	}

       else
	{ gotoxy(20, 15);
	clreol();
	cout << " \n";
	cout << "Invalid Selection, try again               \n";
	}
      }

  // quiting program, disable the MAX1733 and then power down the port
  gotoxy(20, 16);
  clreol();
  cout << " \n";
  cout << "Disabling the MAX1733 Power Supply \n";
  PortData = outp(ControlPort,CP_Disable);  // Disabling the MAX1733
  sound(700);      // alert the user disabling & powering down
  delay(1000);
  cout << " \n";
  cout << "Powering down the port before exiting program \n";
  clreol();
  cout << " \n";
  PortData = 0x00;
  outp(PortAddress,PortData);  // Turn OFF Port Power
  cout << "Port Status = Powered Down \n";
  sound(900);
  delay(1000);
  sound(1100);
  delay(1000);
  nosound();

  return 0;
}
// ******* End of Main Program ******

// *** Functions called by main program ***

// Function receives data from the port
int Receive_In(int statusport)
{
	int data = 0x00;	// clear data storage
	int readmask = 0x20;

      { CLK_Low();
	CS_Low();	// Set chip select low to read parallel data
	CLK_High();	// Clock high to obtain data bit
	CS_High();	// Latched parallel data

	if ((inp(statusport) & readmask) >> 5 == 1) // read MSB, data bit 7
	   data = (data | 0x80);     // Turn on data bit
	else data = (data & 0x7F);   // Turn off data bit

	CLK_Low();
	CLK_High();	// Clock high to obtain next data bit 6
	if ((inp(statusport) & readmask) >> 5 == 1) // read MSB
	   data = (data | 0x40);
	else data = (data & 0xBF);

	CLK_Low();
	CLK_High();	// Clock high to obtain next data bit 5
	if ((inp(statusport) & readmask) >> 5 == 1) // read MSB
	   data = (data | 0x20);
	else data = (data & 0xDF);

	CLK_Low();
	CLK_High();	// Clock high to obtain next data bit 4
	if ((inp(statusport) & readmask) >> 5 == 1) // read MSB
	   data = (data | 0x10);
	else data = (data & 0xEF);

	CLK_Low();
	CLK_High();	// Clock high to obtain next data bit 3
	if ((inp(statusport) & readmask) >> 5 == 1) // read MSB
	   data = (data | 0x08);
	else data = (data & 0xF7);

	CLK_Low();
	CLK_High();	// Clock high to obtain next data bit 2
	if ((inp(statusport) & readmask) >> 5 == 1) // read MSB
	   data = (data | 0x04);
	else data = (data & 0xFB);

	CLK_Low();
	CLK_High();	// Clock high to obtain next data bit 1
	if ((inp(statusport) & readmask) >> 5 == 1) // read MSB
	   data = (data | 0x02);
	else data = (data & 0xFD);

	CLK_Low();
	CLK_High();	// Clock high to obtain next data bit 0
	if ((inp(statusport) & readmask) >> 5 == 1) // read MSB
	   data = (data | 0x01);
	else data = (data & 0xFE);

    }
	return(data);
}

// Optional function to send out data to the port
void Send_Out(int data)
{
	CS_Low();		// Clear present data
	CS_High();
	if ((data & 0x80) >>7 == 1)  // Sending MSB 1st, test if 1 or 0
		Data_High();
	else Data_Low();  	// Data was a 0
	CLK_Low();
	CLK_High();

	if ((data & 0x40) >>6 == 1)  // Sending bit 6
		Data_High();
	else Data_Low();
	CLK_Low();
	CLK_High();

	if ((data & 0x20) >>5 == 1)  // Sending bit 6
		Data_High();
	else Data_Low();
	CLK_Low();
	CLK_High();

	if ((data & 0x10) >>4 == 1)  // Sending bit 5
		Data_High();
	else Data_Low();
	CLK_Low();
	CLK_High();

	if ((data & 0x08) >>3 == 1)  // Sending bit 4
		Data_High();
	else Data_Low();
	CLK_Low();
	CLK_High();

	if ((data & 0x04) >>2 == 1)  // Sending bit 3
		Data_High();
	else Data_Low();
	CLK_Low();
	CLK_High();

	if ((data & 0x02) >>1 == 1)  // Sending bit 2
		Data_High();
	else Data_Low();
	CLK_Low();
	CLK_High();

	if ((data & 0x01) == 1)  // Sending MSB bit 1
		Data_High();
	else Data_Low();
	CLK_Low();
	CLK_High();
// Done sending out the data
}

// Function sets Clock Bit (CLK) LOW
void CLK_Low(void)
{   // Set Clock bit low
	int CLKlow_Mask = 0xFE;   // Mask to turn OFF CLK bit
	PortData = (PortData & CLKlow_Mask); // Set CLK bit low
	outp(PortAddress, PortData);  // Turn OFF CLK bit
}

// Function sets Clock (CLK) HIGH
void CLK_High(void)
{    // Set Clock bit high
	int CLKhigh_Mask = 0x01;  // Mask to turn ON CLK bit
	PortData = (PortData | CLKhigh_Mask); // Set CLK bit high
	outp(PortAddress, PortData);  // Turn ON CLK bit
}

// Function sets Chip Select (CS\) LOW
void CS_Low(void)
{    // Set CS\ bit low
	int CSlow_Mask = 0xFD;   // Mask to turn OFF CS\ bit
	PortData = (PortData & CSlow_Mask); // Set CS\ bit low
	outp(PortAddress, PortData);  // Turn OFF CS bit
}

// Function sets Chip Select (CS\) HIGH
void CS_High(void)
{   // Set CS\ bit high
	int CShigh_Mask = 0x02;  // Mask to turn ON CS\ bit
	PortData = (PortData | CShigh_Mask); // Set CS bit high
	outp(PortAddress, PortData);  // Turn ON CS bit
}

// Optional function for user to send out serial Data
void Data_Low(void)
{    // Set DIN bit low
	int Datalow_Mask = 0xFB;  // Mask to turn OFF Data bit
	PortData = (PortData & Datalow_Mask); // Set Data bit low
	outp(PortAddress, PortData);  // Turn OFF LPT port Data bit
}

// Optional function for user to send out serial Data
void Data_High(void)
{     // Set Data bit high
	int Datahigh_Mask = 0x04; // Mask to turn ON Data bit
	PortData = (PortData | Datahigh_Mask); // Set Data bit high
	outp(PortAddress, PortData);  // Turn ON LPT port Data bit
}
